/**
 *
 * @author <a href="https://oss.lehmann.cx/">Alexander Lehmann</a>
 */

package cx.lehmann.ircbot;

import java.io.IOException;
import java.util.Properties;
import java.util.Random;
import java.util.UUID;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import io.vertx.core.AbstractVerticle;
import io.vertx.core.Promise;
import io.vertx.core.net.NetClient;
import io.vertx.core.net.NetClientOptions;
import io.vertx.core.net.NetSocket;
import io.vertx.core.parsetools.RecordParser;

public class MainVerticle extends AbstractVerticle {

  private final static Logger log = LoggerFactory.getLogger(MainVerticle.class);

  NetSocket netSocket;

  @Override
  public void start(Promise<Void> startPromise) throws Exception {
    Properties prop = new Properties();
    try {
      //load a properties file from class path, inside static method
      prop.load(this.getClass().getClassLoader().getResourceAsStream("config.properties"));
    } 
    catch (IOException ex) {
      ex.printStackTrace();
    }

    NetClientOptions options = new NetClientOptions().setSsl(prop.getProperty("ssl","false").equals("true"));
    NetClient nc = vertx.createNetClient(options);

    nc.connect(Integer.parseInt(prop.getProperty("port")), prop.getProperty("hostname")).onSuccess(
        ns -> {
          log.info("client is connect");
          netSocket=ns;
          startPromise.complete();
          ns.handler(RecordParser.newDelimited("\n").handler(
              msg -> {
                handle(msg.toString());
              }))
          .exceptionHandler(th -> {
            log.warn("got exception", th);
            ns.close(v -> {
              log.info("connection was closed, have to restart", th);
              vertx.close();
            });
          });
          ns.write("CAP LS 302\n"
              + "PASS "+prop.getProperty("password")+"\n"
              + "NICK "+prop.getProperty("nick")+"\n"
              + "USER "+prop.getProperty("user")+" 8 * :"+prop.getProperty("realname")+"\n"
              //              + "CAP REQ :account-notify account-tag away-notify cap-notify extended-join invite-notify message-tags\n"
              //              + "CAP REQ :multi-prefix server-time setname\n"
              + "CAP END\n");
          vertx.setPeriodic(10*1000, 3*60*1000, l -> {
            String random = UUID.randomUUID().toString();
            log.info("PING "+random);
            ns.write("PING "+random+"\n");
          });
        }

        ).onFailure(
            th -> startPromise.fail(th)
            )
        ;

  }

  int currentNumber=0;
  boolean waiterIsRunning=false;

  /**
   * @param string
   */
  private void handle(String string) {


    string=string.substring(0,string.length()-1);

    System.out.println(string);

    if(string.startsWith("PING ")) {
      String pong = string.replaceFirst("PING ", "PONG ");
      netSocket.write(pong);
      netSocket.write("\n");
      log.info(pong);
    }
    else if (string.startsWith(":")) {
      String parts[]=string.split(" ", 4);
      String hostid=parts[0];
      String command=parts.length>1 ? parts[1] : "";
      String channel=parts.length>2 ? parts[2] : "";
      String message=parts.length>3 ? parts[3] : "";
      if(command.equals("PRIVMSG")) {
        if(channel.equals("#counting")) {
          if(message.matches("^[0-9]+$")) {
            System.out.println(message);
            currentNumber=Integer.parseInt(message);
            if(waiterIsRunning) {
              log.info("already waiting");
            } else {
              waiterIsRunning=true;
              log.info("waiting to answer");
              Random rand = new Random();
              vertx.setTimer((60+rand.nextInt(5*60))*1000, l ->{
                String msg="PRIVMSG #counting :"+(currentNumber+1);
                netSocket.write(msg+"\n");
                log.info(msg);
                waiterIsRunning=false;
              });
            }
          }
        }
      }

    }

  }
}
